#include "l3d.h"
#include "sfx.h"
#include "frame.h"
#include "star_system.h"
#include "space.h"
#include "serializer.h"

Sfx::Sfx(void) : Body() {
  m_pos  = vector3d(0, 0, 0);
  m_type = TYPE_EXPLOSION;
  m_age  = 0;
}

void Sfx::Save(void) {
  using namespace Serializer::Write;
  Body::Save();
  wr_vector3d(m_pos);
  wr_float(m_age);
  wr_int(m_type);
}

void Sfx::Load(void) {
  using namespace Serializer::Read;
  Body::Load();
  m_pos = rd_vector3d();
  m_age = rd_float();
  m_type = static_cast<Sfx::TYPE>(rd_int());
}

void Sfx::SetPosition(vector3d p) {
  m_pos = p;
}

void Sfx::TimeStepUpdate(const float timeStep) {
  m_age += timeStep;

  switch(m_type) {
    case TYPE_EXPLOSION:
      if(m_age > 0.2) Space::KillBody(this);
      break;
  }
}

void Sfx::Render(const Frame* camFrame) {
  matrix4x4d ftran;
  Frame::GetFrameTransform(GetFrame(), camFrame, ftran);
  vector3d fpos = ftran*GetPosition();

  glPushMatrix();
  glTranslatef(fpos.x, fpos.y, fpos.z);

  switch(m_type) {
    case TYPE_EXPLOSION:
      glPushAttrib(GL_LIGHTING_BIT | GL_COLOR_BUFFER_BIT);
      glDisable(GL_LIGHTING);
      glColor3f(1,1,0.5);
      gluSphere(L3D::gluQuadric, 1000*m_age, 20, 20);
      glEnable(GL_BLEND);
      glColor4f(1, 0.5, 0, 0.66);
      gluSphere(L3D::gluQuadric, 1500*m_age, 20, 20);
      glColor4f(1,0,0,0.33);
      gluSphere(L3D::gluQuadric, 2000*m_age, 20, 20);
      break;
  }

  glPopMatrix();
}

void Sfx::Add(const Body* b, TYPE t) {
  Sfx* sfx = new Sfx();
  sfx->m_type = t;
  sfx->SetFrame(b->GetFrame());
  sfx->SetPosition(b->GetPosition());
  Space::AddBody(sfx);
}

